import { Relationship } from './enums';
/**
 * Class to create and manage a relationship group. Useful to manage behavior between Peds.
 */
export declare class RelationshipGroup {
    /**
     * The hash of the relationship group
     */
    private hash;
    /**
     * Create a relationship group. Optionally pass a group hash.
     *
     * @param name Name of the relationship group.
     * @param groupHash Optional custom group hash (default: 0).
     */
    constructor(name: string, groupHash?: number);
    /**
     * Gets the hash of the relationship group.
     *
     * @returns The hash of this object.
     */
    readonly Hash: number;
    /**
     * Get the relationship between two relationship groups.
     *
     * @param targetGroup The other relationship group.
     * @returns The relationship
     */
    GetRelationshipBetweenGroups(targetGroup: RelationshipGroup): Relationship;
    /**
     * Set the relationship group between this relationship group and another one.
     *
     * @param targetGroup The other relationship group.
     * @param relationship The desired relationship.
     * @param biDirectionally Whether the target group should have the same relationship towards this relationship group.
     */
    SetRelationshipBetweenGroups(targetGroup: RelationshipGroup, relationship: Relationship, biDirectionally?: boolean): void;
    /**
     * Clear the relationship between this relationship group and another.
     *
     * @param targetGroup The other relationship group.
     * @param relationship The desired relationship to clear.
     * @param biDirectionally Whether the target group should also clear the relationship.
     */
    ClearRelationshipBetweenGroups(targetGroup: RelationshipGroup, relationship: Relationship, biDirectionally?: boolean): void;
    /**
     * Remove this relationship group from the game. This will not delete this object.
     */
    Remove(): void;
}
