import { Entity, Model } from './';
import { Camera } from './Camera';
import { CloudHat, IntersectOptions, MarkerType, Weather } from './enums';
import { Ped, Vehicle } from './models';
import { RaycastResult } from './Raycast';
import { Color, Vector3 } from './utils';
/**
 * Class with common world manipulations.
 *
 * This class includes methods for creating entities and common world rendering.
 */
export declare abstract class World {
    static RenderingCamera: Camera;
    static CurrentDate: Date;
    static Blackout: boolean;
    static CloudHat: CloudHat;
    static CloudHatOpacity: number;
    static Weather: Weather;
    static NextWeather: Weather;
    /**
     * Doesn't work
     */
    /**
    * Doesn't work
    */
    static WeatherTransition: [string | Weather, string | Weather, number];
    static TransitionToWeather(weather: Weather, duration: number): void;
    static DestroyAllCameras(): void;
    /**
     * Create a ped at a desired location.
     *
     * @param model Ped model to be spawned.
     * @param position World position (coordinates) of Ped spawn.
     * @param heading Heading of Ped when spawning.
     */
    static CreatePed(model: Model, position: Vector3, heading?: number): Promise<Ped>;
    /**
     * Create a vehicle at a desired location.
     *
     * @param model Vehicle model to be spawned.
     * @param position World position (coordinates) of Vehicle spawn.
     * @param heading Heading of Vehicle when spawning.
     */
    static CreateVehicle(model: Model, position: Vector3, heading?: number): Promise<Vehicle>;
    /**
     * Draw a marker at a desired location.
     *
     * @param type Type of marker.
     * @param position Location of marker.
     * @param direction Direction facing.
     * @param rotation World rotation.
     * @param scale Size of marker.
     * @param color Color of marker.
     * @param bobUpAndDown Animated movement along marker's own X axis.
     * @param faceCamera Rendering marker facing rendering camera.
     * @param rotateY Rotate along Y axis.
     * @param textureDict Custom texture dictionary for custom marker.
     * @param textureName Custom texture name for custom marker.
     * @param drawOnEntity Render the marker on an entity.
     */
    static DrawMarker(type: MarkerType, position: Vector3, direction: Vector3, rotation: Vector3, scale: Vector3, color: Color, bobUpAndDown?: boolean, faceCamera?: boolean, rotateY?: boolean, textureDict?: string, textureName?: string, drawOnEntity?: boolean): void;
    static Raycast(source: Vector3, direction: Vector3, maxDistance: number, options: IntersectOptions, ignoreEntity: Entity): RaycastResult;
    private static currentCloudHat;
    private static CloudHatDict;
    private static WeatherDict;
}
