"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const __1 = require("../");
const __2 = require("../../");
const enums_1 = require("../../enums");
const Game_1 = require("../../Game");
const utils_1 = require("../../utils");
const items_1 = require("./items");
const modules_1 = require("./modules");
class Menu {
    constructor(title, subtitle, offset, spriteLibrary, spriteName) {
        this.Id = utils_1.UUIDV4();
        this.WidthOffset = 0;
        this.Visible = false;
        this.MouseControlsEnabled = false;
        this.AUDIO_LIBRARY = 'HUD_FRONTEND_DEFAULT_SOUNDSET';
        this.AUDIO_UPDOWN = 'NAV_UP_DOWN';
        this.AUDIO_LEFTRIGHT = 'NAV_LEFT_RIGHT';
        this.AUDIO_SELECT = 'SELECT';
        this.AUDIO_BACK = 'BACK';
        this.AUDIO_ERROR = 'ERROR';
        this.MenuItems = [];
        this.IndexChange = new utils_1.LiteEvent();
        this.ListChange = new utils_1.LiteEvent();
        this.SliderChange = new utils_1.LiteEvent();
        this.SliderSelect = new utils_1.LiteEvent();
        this.CheckboxChange = new utils_1.LiteEvent();
        this.ItemSelect = new utils_1.LiteEvent();
        this.MenuOpen = new utils_1.LiteEvent();
        this.MenuClose = new utils_1.LiteEvent();
        this.MenuChange = new utils_1.LiteEvent();
        this.counterPretext = '';
        this.counterOverride = undefined;
        this.lastUpDownNavigation = 0;
        this.lastLeftRightNavigation = 0;
        this.activeItem = 1000;
        this.extraOffset = 0;
        this.justOpened = true;
        this.safezoneOffset = new utils_1.Point(0, 0);
        this.MaxItemsOnScreen = 9;
        this.maxItem = this.MaxItemsOnScreen;
        this.MouseEdgeEnabled = true;
        if (!(offset instanceof utils_1.Point)) {
            offset = utils_1.Point.Parse(offset);
        }
        this.title = title;
        this.subtitle = subtitle;
        this.spriteLibrary = spriteLibrary || 'commonmenu';
        this.spriteName = spriteName || 'interaction_bgd';
        this.offset = new utils_1.Point(offset.X, offset.Y);
        this.Children = new Map();
        // Create everything
        this.mainMenu = new __1.Container(new utils_1.Point(0, 0), new utils_1.Size(700, 500), new utils_1.Color(0, 0, 0, 0));
        this.logo = new __1.Sprite(this.spriteLibrary, this.spriteName, new utils_1.Point(0 + this.offset.X, 0 + this.offset.Y), new utils_1.Size(431, 107));
        this.mainMenu.addItem((this.titleResText = new modules_1.ResText(this.title, new utils_1.Point(215 + this.offset.X, 20 + this.offset.Y), 1.15, new utils_1.Color(255, 255, 255, 255), 1, enums_1.Alignment.Centered)));
        if (this.subtitle !== '') {
            this.mainMenu.addItem(new modules_1.ResRectangle(new utils_1.Point(0 + this.offset.X, 107 + this.offset.Y), new utils_1.Size(431, 37), new utils_1.Color(255, 0, 0, 0)));
            this.mainMenu.addItem((this.subtitleResText = new modules_1.ResText(this.subtitle, new utils_1.Point(8 + this.offset.X, 110 + this.offset.Y), 0.35, new utils_1.Color(255, 255, 255, 255), 0, enums_1.Alignment.Left)));
            if (this.subtitle.startsWith('~')) {
                this.counterPretext = this.subtitle.substr(0, 3);
            }
            this.counterText = new modules_1.ResText('', new utils_1.Point(425 + this.offset.X, 110 + this.offset.Y), 0.35, new utils_1.Color(255, 255, 255, 255), 0, enums_1.Alignment.Right);
            this.extraOffset += 37;
        }
        this.upAndDownSprite = new __1.Sprite('commonmenu', 'shop_arrows_upanddown', new utils_1.Point(190 + this.offset.X, 147 + 37 * (this.MaxItemsOnScreen + 1) + this.offset.Y - 37 + this.extraOffset), new utils_1.Size(50, 50));
        this.extraRectangleUp = new modules_1.ResRectangle(new utils_1.Point(0 + this.offset.X, 144 + 38 * (this.MaxItemsOnScreen + 1) + this.offset.Y - 37 + this.extraOffset), new utils_1.Size(431, 18), new utils_1.Color(200, 0, 0, 0));
        this.extraRectangleDown = new modules_1.ResRectangle(new utils_1.Point(0 + this.offset.X, 144 + 18 + 38 * (this.MaxItemsOnScreen + 1) + this.offset.Y - 37 + this.extraOffset), new utils_1.Size(431, 18), new utils_1.Color(200, 0, 0, 0));
        this.descriptionBar = new modules_1.ResRectangle(new utils_1.Point(this.offset.X, 123), new utils_1.Size(431, 4), utils_1.Color.Black);
        this.descriptionRectangle = new __1.Sprite('commonmenu', 'gradient_bgd', new utils_1.Point(this.offset.X, 127), new utils_1.Size(431, 30));
        this.descriptionText = new modules_1.ResText('Description', new utils_1.Point(this.offset.X + 5, 125), 0.35, new utils_1.Color(255, 255, 255, 255), enums_1.Font.ChaletLondon, enums_1.Alignment.Left);
        this.background = new __1.Sprite('commonmenu', 'gradient_bgd', new utils_1.Point(this.offset.X, 144 + this.offset.Y - 37 + this.extraOffset), new utils_1.Size(290, 25));
        setTick(() => {
            this.render();
        });
    }
    get CurrentSelection() {
        return this.activeItem % this.MenuItems.length;
    }
    set CurrentSelection(v) {
        this.MenuItems[this.activeItem % this.MenuItems.length].Selected = false;
        this.activeItem = 1000 - (1000 % this.MenuItems.length) + v;
        if (this.CurrentSelection > this.maxItem) {
            this.maxItem = this.CurrentSelection;
            this.minItem = this.CurrentSelection - this.MaxItemsOnScreen;
        }
        else if (this.CurrentSelection < this.minItem) {
            this.maxItem = this.MaxItemsOnScreen + this.CurrentSelection;
            this.minItem = this.CurrentSelection;
        }
    }
    SetMenuWidthOffset(widthOffset) {
        this.WidthOffset = widthOffset;
        if (this.logo != null) {
            this.logo.size = new utils_1.Size(431 + this.WidthOffset, 107);
        }
        this.mainMenu.Items[0].pos = new utils_1.Point((this.WidthOffset + this.offset.X + 431) / 2, 20 + this.offset.Y);
        if (this.counterText) {
            this.counterText.pos = new utils_1.Point(425 + this.offset.X + widthOffset, 110 + this.offset.Y);
        }
        if (this.mainMenu.Items.length >= 2) {
            const tmp = this.mainMenu.Items[1];
            tmp.size = new utils_1.Size(431 + this.WidthOffset, 37);
        }
    }
    AddItem(item) {
        if (this.justOpened) {
            this.justOpened = false;
        }
        item.Offset = this.offset;
        item.Parent = this;
        item.SetVerticalPosition(this.MenuItems.length * 25 - 37 + this.extraOffset);
        this.MenuItems.push(item);
        item.Description = this.FormatDescription(item.Description);
        this.RefreshIndex();
        this.RecalculateDescriptionPosition();
    }
    RefreshIndex() {
        if (this.MenuItems.length === 0) {
            this.activeItem = 1000;
            this.maxItem = this.MaxItemsOnScreen;
            this.minItem = 0;
            return;
        }
        for (const item of this.MenuItems) {
            item.Selected = false;
        }
        this.activeItem = 1000 - (1000 % this.MenuItems.length);
        this.maxItem = this.MaxItemsOnScreen;
        this.minItem = 0;
    }
    Clear() {
        this.MenuItems = [];
        this.RecalculateDescriptionPosition();
    }
    Open() {
        __2.Audio.PlaySoundFrontEnd(this.AUDIO_BACK, this.AUDIO_LIBRARY);
        this.Visible = true;
        this.justOpened = true;
        this.MenuOpen.emit();
    }
    Close() {
        __2.Audio.PlaySoundFrontEnd(this.AUDIO_BACK, this.AUDIO_LIBRARY);
        this.Visible = false;
        this.RefreshIndex();
        this.MenuClose.emit();
    }
    set Subtitle(text) {
        this.subtitle = text;
        this.subtitleResText.caption = text;
    }
    GoLeft() {
        if (!(this.MenuItems[this.CurrentSelection] instanceof items_1.UIMenuListItem) &&
            !(this.MenuItems[this.CurrentSelection] instanceof items_1.UIMenuSliderItem)) {
            return;
        }
        if (this.MenuItems[this.CurrentSelection] instanceof items_1.UIMenuListItem) {
            const it = this.MenuItems[this.CurrentSelection];
            if (it.Collection.length === 0) {
                return;
            }
            it.Index--;
            __2.Audio.PlaySoundFrontEnd(this.AUDIO_LEFTRIGHT, this.AUDIO_LIBRARY);
            this.ListChange.emit(it, it.Index);
        }
        else if (this.MenuItems[this.CurrentSelection] instanceof items_1.UIMenuSliderItem) {
            const it = this.MenuItems[this.CurrentSelection];
            it.Index = it.Index - 1;
            __2.Audio.PlaySoundFrontEnd(this.AUDIO_LEFTRIGHT, this.AUDIO_LIBRARY);
            this.SliderChange.emit(it, it.Index, it.IndexToItem(it.Index));
            // it.SliderChangedTrigger(it.Index);
        }
    }
    GoRight() {
        if (!(this.MenuItems[this.CurrentSelection] instanceof items_1.UIMenuListItem) &&
            !(this.MenuItems[this.CurrentSelection] instanceof items_1.UIMenuSliderItem)) {
            return;
        }
        if (this.MenuItems[this.CurrentSelection] instanceof items_1.UIMenuListItem) {
            const it = this.MenuItems[this.CurrentSelection];
            if (it.Collection.length === 0) {
                return;
            }
            it.Index++;
            __2.Audio.PlaySoundFrontEnd(this.AUDIO_LEFTRIGHT, this.AUDIO_LIBRARY);
            this.ListChange.emit(it, it.Index);
        }
        else if (this.MenuItems[this.CurrentSelection] instanceof items_1.UIMenuSliderItem) {
            const it = this.MenuItems[this.CurrentSelection];
            it.Index++;
            __2.Audio.PlaySoundFrontEnd(this.AUDIO_LEFTRIGHT, this.AUDIO_LIBRARY);
            this.SliderChange.emit(it, it.Index, it.IndexToItem(it.Index));
            // it.SliderChangedTrigger(it.Index);
        }
    }
    SelectItem() {
        if (!this.MenuItems[this.CurrentSelection].Enabled) {
            __2.Audio.PlaySoundFrontEnd(this.AUDIO_ERROR, this.AUDIO_LIBRARY);
            return;
        }
        const it = this.MenuItems[this.CurrentSelection];
        if (this.MenuItems[this.CurrentSelection] instanceof items_1.UIMenuCheckboxItem) {
            it.Checked = !it.Checked;
            __2.Audio.PlaySoundFrontEnd(this.AUDIO_SELECT, this.AUDIO_LIBRARY);
            this.CheckboxChange.emit(it, it.Checked);
        }
        else {
            __2.Audio.PlaySoundFrontEnd(this.AUDIO_SELECT, this.AUDIO_LIBRARY);
            this.ItemSelect.emit(it, this.CurrentSelection);
            if (this.Children.has(it.Id)) {
                const subMenu = this.Children.get(it.Id);
                this.Visible = false;
                subMenu.Visible = true;
                subMenu.justOpened = true;
                subMenu.MenuOpen.emit();
                this.MenuChange.emit(subMenu, true);
            }
        }
        it.fireEvent();
    }
    // public getMousePosition(relative: boolean = false) {
    //   const screenw = Screen.Width;
    //   const screenh = Screen.Height;
    //   const cursor = mp.gui.cursor.position;
    //   let [mouseX, mouseY] = [cursor[0], cursor[1]];
    //   if (relative) [mouseX, mouseY] = [cursor[0] / screenw, cursor[1] / screenh];
    //   return [mouseX, mouseY];
    // }
    GetScreenResolutionMantainRatio() {
        const height = __1.Screen.Height;
        const width = __1.Screen.ScaledWidth;
        return new utils_1.Size(width, height);
    }
    // public IsMouseInBounds(topLeft: Point, boxSize: Size) {
    //   const res = this.GetScreenResolutionMantainRatio();
    //   const [mouseX, mouseY] = this.getMousePosition();
    //   return (
    //     mouseX >= topLeft.X &&
    //     mouseX <= topLeft.X + boxSize.Width &&
    //     (mouseY > topLeft.Y && mouseY < topLeft.Y + boxSize.Height)
    //   );
    // }
    // public IsMouseInListItemArrows(
    //   item,
    //   topLeft,
    //   safezone, // TODO: Ability to scroll left and right
    // ) {
    //   BeginTextCommandWidth('jamyfafi');
    //   AddTextComponentSubstringPlayerName(item.Text);
    //   var res = this.GetScreenResolutionMantainRatio();
    //   var screenw = res.Width;
    //   var screenh = res.Height;
    //   const height = 1080.0;
    //   const ratio = screenw / screenh;
    //   var width = height * ratio;
    //   const labelSize = EndTextCommandGetWidth(false) * width * 0.35;
    //   const labelSizeX = 5 + labelSize + 10;
    //   const arrowSizeX = 431 - labelSizeX;
    //   return this.IsMouseInBounds(topLeft, new Size(labelSizeX, 38))
    //     ? 1
    //     : this.IsMouseInBounds(new Point(topLeft.X + labelSizeX, topLeft.Y), new Size(arrowSizeX, 38))
    //     ? 2
    //     : 0;
    // }
    // public ProcessMouse() {
    //   if (!this.Visible || this.justOpened || this.MenuItems.length == 0 || !this.MouseControlsEnabled) {
    //     /*Common.EnableControl(0, GameControl.LookUpDown);
    //               Common.EnableControl(0, GameControl.LookLeftRight);
    //               Common.EnableControl(0, GameControl.Aim);
    // 			Common.EnableControl(0, GameControl.Attack);*/
    //     this.MenuItems.filter(i => i.Hovered).forEach(i => (i.Hovered = false));
    //     return;
    //   }
    //   if (!mp.gui.cursor.visible) mp.gui.cursor.visible = true;
    //   let limit = this.MenuItems.length - 1;
    //   let counter = 0;
    //   if (this.MenuItems.length > this.MaxItemsOnScreen + 1) limit = this.maxItem;
    //   if (this.IsMouseInBounds(new Point(0, 0), new Size(30, 1080)) && this.MouseEdgeEnabled) {
    //     SetGameplayCamRelativeHeading(GetGameplayCamRelativeHeading() + 5.0);
    //     SetCursorSprite(6);
    //   } else if (
    //     this.IsMouseInBounds(new Point(this.GetScreenResolutionMantainRatio().Width - 30.0, 0), new Size(30, 1080)) &&
    //     this.MouseEdgeEnabled
    //   ) {
    //     SetGameplayCamRelativeHeading(GetGameplayCamRelativeHeading() - 5.0);
    //     SetCursorSprite(7);
    //   } else if (this.MouseEdgeEnabled) {
    //     SetCursorSprite(1);
    //   }
    //   for (let i = this.minItem; i <= limit; i++) {
    //     let xpos = this.offset.X;
    //     let ypos = this.offset.Y + 144 - 37 + this.extraOffset + counter * 38;
    //     let xsize = 431 + this.WidthOffset;
    //     const ysize = 38;
    //     const uiMenuItem = this.MenuItems[i];
    //     if (this.IsMouseInBounds(new Point(xpos, ypos), new Size(xsize, ysize))) {
    //       uiMenuItem.Hovered = true;
    //       if (Game.IsControlJustPressed(0, Control.Attack) || Game.IsDisabledControlJustPressed(0, Control.Attack))
    //         if (uiMenuItem.Selected && uiMenuItem.Enabled) {
    //           if (
    //             this.MenuItems[i] instanceof UIMenuListItem &&
    //             this.IsMouseInListItemArrows(this.MenuItems[i], new Point(xpos, ypos), 0) > 0
    //           ) {
    //             const res = this.IsMouseInListItemArrows(this.MenuItems[i], new Point(xpos, ypos), 0);
    //             switch (res) {
    //               case 1:
    //                 Audio.PlaySoundFrontEnd(this.AUDIO_SELECT, this.AUDIO_LIBRARY);
    //                 //this.MenuItems[i].ItemActivate(this);
    //                 this.MenuItems[i].fireEvent();
    //                 this.ItemSelect.emit(this.MenuItems[i], i);
    //                 break;
    //               case 2:
    //                 var it = <any>this.MenuItems[i];
    //                 if ((it.Collection == null ? it.Items.Count : it.Collection.Count) > 0) {
    //                   it.Index++;
    //                   Audio.PlaySoundFrontEnd(this.AUDIO_LEFTRIGHT, this.AUDIO_LIBRARY);
    //                   this.ListChange.emit(it, it.Index);
    //                 }
    //                 break;
    //             }
    //           } else this.SelectItem();
    //         } else if (!uiMenuItem.Selected) {
    //           this.CurrentSelection = i;
    //           Audio.PlaySoundFrontEnd(this.AUDIO_UPDOWN, this.AUDIO_LIBRARY);
    //           this.IndexChange.emit(this.CurrentSelection);
    //           this.SelectItem();
    //         } else if (!uiMenuItem.Enabled && uiMenuItem.Selected) {
    //           Audio.PlaySoundFrontEnd(this.AUDIO_ERROR, this.AUDIO_LIBRARY);
    //         }
    //     } else uiMenuItem.Hovered = false;
    //     counter++;
    //   }
    //   const extraY =
    //     144 + 38 * (this.MaxItemsOnScreen + 1) + this.offset.Y - 37 + this.extraOffset + this.safezoneOffset.Y;
    //   const extraX = this.safezoneOffset.X + this.offset.X;
    //   if (this.MenuItems.length <= this.MaxItemsOnScreen + 1) return;
    //   if (this.IsMouseInBounds(new Point(extraX, extraY), new Size(431 + this.WidthOffset, 18))) {
    //     this.extraRectangleUp.color = new Color(30, 30, 30, 255);
    //     if (Game.IsControlJustPressed(0, Control.Attack) || Game.IsDisabledControlJustPressed(0, Control.Attack)) {
    //       if (this.MenuItems.length > this.MaxItemsOnScreen + 1) this.GoUpOverflow();
    //       else this.GoUp();
    //     }
    //   } else this.extraRectangleUp.color = new Color(0, 0, 0, 200);
    //   if (this.IsMouseInBounds(new Point(extraX, extraY + 18), new Size(431 + this.WidthOffset, 18))) {
    //     this.extraRectangleDown.color = new Color(30, 30, 30, 255);
    //     if (Game.IsControlJustPressed(0, Control.Attack) || Game.IsDisabledControlJustPressed(0, Control.Attack)) {
    //       if (this.MenuItems.length > this.MaxItemsOnScreen + 1) this.GoDownOverflow();
    //       else this.GoDown();
    //     }
    //   } else this.extraRectangleDown.color = new Color(0, 0, 0, 200);
    // }
    ProcessControl() {
        if (!this.Visible) {
            return;
        }
        if (this.justOpened) {
            this.justOpened = false;
            return;
        }
        if (Game_1.Game.IsControlJustReleased(0, enums_1.Control.PhoneCancel)) {
            // Back
            this.GoBack();
        }
        if (this.MenuItems.length === 0) {
            return;
        }
        if (Game_1.Game.IsControlPressed(0, enums_1.Control.PhoneUp) && this.lastUpDownNavigation + 200 < Date.now()) {
            // Up
            this.lastUpDownNavigation = Date.now();
            if (this.MenuItems.length > this.MaxItemsOnScreen + 1) {
                this.GoUpOverflow();
            }
            else {
                this.GoUp();
            }
        }
        else if (Game_1.Game.IsControlPressed(0, enums_1.Control.PhoneDown) && this.lastUpDownNavigation + 200 < Date.now()) {
            // Down
            this.lastUpDownNavigation = Date.now();
            if (this.MenuItems.length > this.MaxItemsOnScreen + 1) {
                this.GoDownOverflow();
            }
            else {
                this.GoDown();
            }
        }
        else if (Game_1.Game.IsControlPressed(0, enums_1.Control.PhoneLeft) && this.lastLeftRightNavigation + 200 < Date.now()) {
            // Left
            this.lastLeftRightNavigation = Date.now();
            this.GoLeft();
        }
        else if (Game_1.Game.IsControlPressed(0, enums_1.Control.PhoneRight) && this.lastLeftRightNavigation + 200 < Date.now()) {
            // Right
            this.lastLeftRightNavigation = Date.now();
            this.GoRight();
        }
        else if (Game_1.Game.IsControlJustPressed(0, enums_1.Control.FrontendAccept)) {
            // Select
            this.SelectItem();
        }
    }
    GoUpOverflow() {
        if (this.MenuItems.length <= this.MaxItemsOnScreen + 1) {
            return;
        }
        if (this.activeItem % this.MenuItems.length <= this.minItem) {
            if (this.activeItem % this.MenuItems.length === 0) {
                this.minItem = this.MenuItems.length - this.MaxItemsOnScreen - 1;
                this.maxItem = this.MenuItems.length - 1;
                this.MenuItems[this.activeItem % this.MenuItems.length].Selected = false;
                this.activeItem = 1000 - (1000 % this.MenuItems.length);
                this.activeItem += this.MenuItems.length - 1;
                this.MenuItems[this.activeItem % this.MenuItems.length].Selected = true;
            }
            else {
                this.minItem--;
                this.maxItem--;
                this.MenuItems[this.activeItem % this.MenuItems.length].Selected = false;
                this.activeItem--;
                this.MenuItems[this.activeItem % this.MenuItems.length].Selected = true;
            }
        }
        else {
            this.MenuItems[this.activeItem % this.MenuItems.length].Selected = false;
            this.activeItem--;
            this.MenuItems[this.activeItem % this.MenuItems.length].Selected = true;
        }
        __2.Audio.PlaySoundFrontEnd(this.AUDIO_UPDOWN, this.AUDIO_LIBRARY);
        this.IndexChange.emit(this.CurrentSelection);
    }
    GoUp() {
        if (this.MenuItems.length > this.MaxItemsOnScreen + 1) {
            return;
        }
        this.MenuItems[this.activeItem % this.MenuItems.length].Selected = false;
        this.activeItem--;
        this.MenuItems[this.activeItem % this.MenuItems.length].Selected = true;
        __2.Audio.PlaySoundFrontEnd(this.AUDIO_UPDOWN, this.AUDIO_LIBRARY);
        this.IndexChange.emit(this.CurrentSelection);
    }
    GoDownOverflow() {
        if (this.MenuItems.length <= this.MaxItemsOnScreen + 1) {
            return;
        }
        if (this.activeItem % this.MenuItems.length >= this.maxItem) {
            if (this.activeItem % this.MenuItems.length === this.MenuItems.length - 1) {
                this.minItem = 0;
                this.maxItem = this.MaxItemsOnScreen;
                this.MenuItems[this.activeItem % this.MenuItems.length].Selected = false;
                this.activeItem = 1000 - (1000 % this.MenuItems.length);
                this.MenuItems[this.activeItem % this.MenuItems.length].Selected = true;
            }
            else {
                this.minItem++;
                this.maxItem++;
                this.MenuItems[this.activeItem % this.MenuItems.length].Selected = false;
                this.activeItem++;
                this.MenuItems[this.activeItem % this.MenuItems.length].Selected = true;
            }
        }
        else {
            this.MenuItems[this.activeItem % this.MenuItems.length].Selected = false;
            this.activeItem++;
            this.MenuItems[this.activeItem % this.MenuItems.length].Selected = true;
        }
        __2.Audio.PlaySoundFrontEnd(this.AUDIO_UPDOWN, this.AUDIO_LIBRARY);
        this.IndexChange.emit(this.CurrentSelection);
    }
    GoDown() {
        if (this.MenuItems.length > this.MaxItemsOnScreen + 1) {
            return;
        }
        this.MenuItems[this.activeItem % this.MenuItems.length].Selected = false;
        this.activeItem++;
        this.MenuItems[this.activeItem % this.MenuItems.length].Selected = true;
        __2.Audio.PlaySoundFrontEnd(this.AUDIO_UPDOWN, this.AUDIO_LIBRARY);
        this.IndexChange.emit(this.CurrentSelection);
    }
    GoBack() {
        __2.Audio.PlaySoundFrontEnd(this.AUDIO_BACK, this.AUDIO_LIBRARY);
        this.Visible = false;
        if (this.ParentMenu != null) {
            this.ParentMenu.Visible = true;
            this.ParentMenu.justOpened = true;
            this.ParentMenu.MenuOpen.emit();
            this.MenuChange.emit(this.ParentMenu, false);
        }
        this.MenuClose.emit();
    }
    BindMenuToItem(menuToBind, itemToBindTo) {
        menuToBind.ParentMenu = this;
        menuToBind.ParentItem = itemToBindTo;
        this.Children.set(itemToBindTo.Id, menuToBind);
    }
    ReleaseMenuFromItem(releaseFrom) {
        if (!this.Children.has(releaseFrom.Id)) {
            return false;
        }
        const menu = this.Children.get(releaseFrom.Id);
        menu.ParentItem = null;
        menu.ParentMenu = null;
        this.Children.delete(releaseFrom.Id);
        return true;
    }
    RecalculateDescriptionPosition() {
        this.descriptionBar.pos = new utils_1.Point(this.offset.X, 149 - 37 + this.extraOffset + this.offset.Y);
        this.descriptionRectangle.pos = new utils_1.Point(this.offset.X, 149 - 37 + this.extraOffset + this.offset.Y);
        this.descriptionText.pos = new utils_1.Point(this.offset.X + 8, 155 - 37 + this.extraOffset + this.offset.Y);
        this.descriptionBar.size = new utils_1.Size(431 + this.WidthOffset, 4);
        this.descriptionRectangle.size = new utils_1.Size(431 + this.WidthOffset, 30);
        let count = this.MenuItems.length;
        if (count > this.MaxItemsOnScreen + 1) {
            count = this.MaxItemsOnScreen + 2;
        }
        this.descriptionBar.pos = new utils_1.Point(this.offset.X, 38 * count + this.descriptionBar.pos.Y);
        this.descriptionRectangle.pos = new utils_1.Point(this.offset.X, 38 * count + this.descriptionRectangle.pos.Y);
        this.descriptionText.pos = new utils_1.Point(this.offset.X + 8, 38 * count + this.descriptionText.pos.Y);
    }
    FormatDescription(input) {
        if (input.length > 99) {
            input = input.slice(0, 99);
        }
        const maxPixelsPerLine = 425 + this.WidthOffset;
        let aggregatePixels = 0;
        let output = '';
        const words = input.split(' ');
        for (const word of words) {
            const offset = utils_1.MeasureString(word);
            aggregatePixels += offset;
            if (aggregatePixels > maxPixelsPerLine) {
                output += '\n' + word + ' ';
                aggregatePixels = offset + utils_1.MeasureString(' ');
            }
            else {
                output += word + ' ';
                aggregatePixels += utils_1.MeasureString(' ');
            }
        }
        return output;
    }
    render() {
        if (!this.Visible) {
            return;
        }
        if (this.justOpened) {
            if (this.logo != null && !this.logo.IsTextureDictionaryLoaded) {
                this.logo.LoadTextureDictionary();
            }
            if (!this.background.IsTextureDictionaryLoaded) {
                this.background.LoadTextureDictionary();
            }
            if (!this.descriptionRectangle.IsTextureDictionaryLoaded) {
                this.descriptionRectangle.LoadTextureDictionary();
            }
            if (!this.upAndDownSprite.IsTextureDictionaryLoaded) {
                this.upAndDownSprite.LoadTextureDictionary();
            }
        }
        this.mainMenu.Draw();
        // this.ProcessMouse();
        this.ProcessControl();
        this.background.size =
            this.MenuItems.length > this.MaxItemsOnScreen + 1
                ? new utils_1.Size(431 + this.WidthOffset, 38 * (this.MaxItemsOnScreen + 1))
                : new utils_1.Size(431 + this.WidthOffset, 38 * this.MenuItems.length);
        this.background.Draw();
        if (this.MenuItems.length > 0) {
            this.MenuItems[this.activeItem % this.MenuItems.length].Selected = true;
            if (this.MenuItems[this.activeItem % this.MenuItems.length].Description.trim() !== '') {
                this.RecalculateDescriptionPosition();
                const descCaption = this.MenuItems[this.activeItem % this.MenuItems.length].Description;
                // descCaption = this.FormatDescription(descCaption);
                this.descriptionText.caption = descCaption;
                const numLines = this.descriptionText.caption.split('\n').length;
                this.descriptionRectangle.size = new utils_1.Size(431 + this.WidthOffset, numLines * 25 + 15);
                this.descriptionBar.Draw();
                this.descriptionRectangle.Draw();
                this.descriptionText.Draw();
            }
        }
        if (this.MenuItems.length <= this.MaxItemsOnScreen + 1) {
            let count = 0;
            for (const menuItem of this.MenuItems) {
                menuItem.SetVerticalPosition(count * 38 - 37 + this.extraOffset);
                menuItem.Draw();
                count++;
            }
            if (this.counterText && this.counterOverride) {
                this.counterText.caption = this.counterPretext + this.counterOverride;
                this.counterText.Draw();
            }
        }
        else {
            let count = 0;
            for (let index = this.minItem; index <= this.maxItem; index++) {
                const item = this.MenuItems[index];
                item.SetVerticalPosition(count * 38 - 37 + this.extraOffset);
                item.Draw();
                count++;
            }
            this.extraRectangleUp.size = new utils_1.Size(431 + this.WidthOffset, 18);
            this.extraRectangleDown.size = new utils_1.Size(431 + this.WidthOffset, 18);
            this.upAndDownSprite.pos = new utils_1.Point(190 + this.offset.X + this.WidthOffset / 2, 147 + 37 * (this.MaxItemsOnScreen + 1) + this.offset.Y - 37 + this.extraOffset);
            this.extraRectangleUp.Draw();
            this.extraRectangleDown.Draw();
            this.upAndDownSprite.Draw();
            if (this.counterText) {
                if (!this.counterOverride) {
                    const cap = this.CurrentSelection + 1 + ' / ' + this.MenuItems.length;
                    this.counterText.caption = this.counterPretext + cap;
                }
                else {
                    this.counterText.caption = this.counterPretext + this.counterOverride;
                }
                this.counterText.Draw();
            }
        }
        this.logo.Draw();
    }
}
exports.Menu = Menu;
