"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const __1 = require("../../");
const enums_1 = require("../../../enums");
const utils_1 = require("../../../utils");
const modules_1 = require("../modules/");
class UIMenuItem {
    constructor(text, description = '') {
        this.Id = utils_1.UUIDV4();
        this.BackColor = UIMenuItem.DefaultBackColor;
        this.HighlightedBackColor = UIMenuItem.DefaultHighlightedBackColor;
        this.ForeColor = UIMenuItem.DefaultForeColor;
        this.HighlightedForeColor = UIMenuItem.DefaultHighlightedForeColor;
        this.RightLabel = '';
        this.LeftBadge = enums_1.BadgeStyle.None;
        this.RightBadge = enums_1.BadgeStyle.None;
        this.Enabled = true;
        this.rectangle = new modules_1.ResRectangle(new utils_1.Point(0, 0), new utils_1.Size(431, 38), new utils_1.Color(150, 0, 0, 0));
        this.text = new modules_1.ResText(text, new utils_1.Point(8, 0), 0.33, utils_1.Color.WhiteSmoke, enums_1.Font.ChaletLondon, enums_1.Alignment.Left);
        this.Description = description;
        this.selectedSprite = new __1.Sprite('commonmenu', 'gradient_nav', new utils_1.Point(0, 0), new utils_1.Size(431, 38));
        this.badgeLeft = new __1.Sprite('commonmenu', '', new utils_1.Point(0, 0), new utils_1.Size(40, 40));
        this.badgeRight = new __1.Sprite('commonmenu', '', new utils_1.Point(0, 0), new utils_1.Size(40, 40));
        this.labelText = new modules_1.ResText('', new utils_1.Point(0, 0), 0.35, utils_1.Color.White, 0, enums_1.Alignment.Right);
    }
    get Text() {
        return this.text.caption;
    }
    set Text(v) {
        this.text.caption = v;
    }
    SetVerticalPosition(y) {
        this.rectangle.pos = new utils_1.Point(this.Offset.X, y + 144 + this.Offset.Y);
        this.selectedSprite.pos = new utils_1.Point(0 + this.Offset.X, y + 144 + this.Offset.Y);
        this.text.pos = new utils_1.Point(8 + this.Offset.X, y + 147 + this.Offset.Y);
        this.badgeLeft.pos = new utils_1.Point(0 + this.Offset.X, y + 142 + this.Offset.Y);
        this.badgeRight.pos = new utils_1.Point(385 + this.Offset.X, y + 142 + this.Offset.Y);
        this.labelText.pos = new utils_1.Point(420 + this.Offset.X, y + 148 + this.Offset.Y);
    }
    addEvent(event, ...args) {
        this.event = { event, args };
    }
    fireEvent() {
        if (this.event) {
            TriggerEvent(this.event.event, ...this.event.args);
        }
    }
    Draw() {
        this.rectangle.size = new utils_1.Size(431 + this.Parent.WidthOffset, 38);
        this.selectedSprite.size = new utils_1.Size(431 + this.Parent.WidthOffset, 38);
        if (this.Hovered && !this.Selected) {
            this.rectangle.color = new utils_1.Color(20, 255, 255, 255);
            this.rectangle.Draw();
        }
        this.selectedSprite.color = this.Selected ? this.HighlightedBackColor : this.BackColor;
        this.selectedSprite.Draw();
        this.text.color = this.Enabled
            ? this.Selected
                ? this.HighlightedForeColor
                : this.ForeColor
            : new utils_1.Color(255, 163, 159, 148);
        if (this.LeftBadge !== enums_1.BadgeStyle.None) {
            this.text.pos = new utils_1.Point(35 + this.Offset.X, this.text.pos.Y);
            this.badgeLeft.TextureDict = this.BadgeToSpriteLib(this.LeftBadge);
            this.badgeLeft.textureName = this.BadgeToSpriteName(this.LeftBadge, this.Selected);
            this.badgeLeft.color = this.IsBagdeWhiteSprite(this.LeftBadge)
                ? this.Enabled
                    ? this.Selected
                        ? this.HighlightedForeColor
                        : this.ForeColor
                    : new utils_1.Color(255, 163, 159, 148)
                : utils_1.Color.White;
            this.badgeLeft.Draw();
        }
        else {
            this.text.pos = new utils_1.Point(8 + this.Offset.X, this.text.pos.Y);
        }
        if (this.RightBadge !== enums_1.BadgeStyle.None) {
            this.badgeRight.pos = new utils_1.Point(385 + this.Offset.X + this.Parent.WidthOffset, this.badgeRight.pos.Y);
            this.badgeRight.TextureDict = this.BadgeToSpriteLib(this.RightBadge);
            this.badgeRight.textureName = this.BadgeToSpriteName(this.RightBadge, this.Selected);
            this.badgeRight.color = this.IsBagdeWhiteSprite(this.RightBadge)
                ? this.Enabled
                    ? this.Selected
                        ? this.HighlightedForeColor
                        : this.ForeColor
                    : new utils_1.Color(255, 163, 159, 148)
                : utils_1.Color.White;
            this.badgeRight.Draw();
        }
        if (this.RightLabel && this.RightLabel !== '') {
            this.labelText.pos = new utils_1.Point(420 + this.Offset.X + this.Parent.WidthOffset, this.labelText.pos.Y);
            this.labelText.caption = this.RightLabel;
            this.labelText.color = this.text.color = this.Enabled
                ? this.Selected
                    ? this.HighlightedForeColor
                    : this.ForeColor
                : new utils_1.Color(255, 163, 159, 148);
            this.labelText.Draw();
        }
        this.text.Draw();
    }
    SetLeftBadge(badge) {
        this.LeftBadge = badge;
    }
    SetRightBadge(badge) {
        this.RightBadge = badge;
    }
    SetRightLabel(text) {
        this.RightLabel = text;
    }
    BadgeToSpriteLib(badge) {
        return 'commonmenu';
    }
    BadgeToSpriteName(badge, selected) {
        switch (badge) {
            case enums_1.BadgeStyle.None:
                return '';
            case enums_1.BadgeStyle.BronzeMedal:
                return 'mp_medal_bronze';
            case enums_1.BadgeStyle.GoldMedal:
                return 'mp_medal_gold';
            case enums_1.BadgeStyle.SilverMedal:
                return 'medal_silver';
            case enums_1.BadgeStyle.Alert:
                return 'mp_alerttriangle';
            case enums_1.BadgeStyle.Crown:
                return 'mp_hostcrown';
            case enums_1.BadgeStyle.Ammo:
                return selected ? 'shop_ammo_icon_b' : 'shop_ammo_icon_a';
            case enums_1.BadgeStyle.Armour:
                return selected ? 'shop_armour_icon_b' : 'shop_armour_icon_a';
            case enums_1.BadgeStyle.Barber:
                return selected ? 'shop_barber_icon_b' : 'shop_barber_icon_a';
            case enums_1.BadgeStyle.Clothes:
                return selected ? 'shop_clothing_icon_b' : 'shop_clothing_icon_a';
            case enums_1.BadgeStyle.Franklin:
                return selected ? 'shop_franklin_icon_b' : 'shop_franklin_icon_a';
            case enums_1.BadgeStyle.Bike:
                return selected ? 'shop_garage_bike_icon_b' : 'shop_garage_bike_icon_a';
            case enums_1.BadgeStyle.Car:
                return selected ? 'shop_garage_icon_b' : 'shop_garage_icon_a';
            case enums_1.BadgeStyle.Gun:
                return selected ? 'shop_gunclub_icon_b' : 'shop_gunclub_icon_a';
            case enums_1.BadgeStyle.Heart:
                return selected ? 'shop_health_icon_b' : 'shop_health_icon_a';
            case enums_1.BadgeStyle.Lock:
                return 'shop_lock';
            case enums_1.BadgeStyle.Makeup:
                return selected ? 'shop_makeup_icon_b' : 'shop_makeup_icon_a';
            case enums_1.BadgeStyle.Mask:
                return selected ? 'shop_mask_icon_b' : 'shop_mask_icon_a';
            case enums_1.BadgeStyle.Michael:
                return selected ? 'shop_michael_icon_b' : 'shop_michael_icon_a';
            case enums_1.BadgeStyle.Star:
                return 'shop_new_star';
            case enums_1.BadgeStyle.Tatoo:
                return selected ? 'shop_tattoos_icon_b' : 'shop_tattoos_icon_';
            case enums_1.BadgeStyle.Tick:
                return 'shop_tick_icon';
            case enums_1.BadgeStyle.Trevor:
                return selected ? 'shop_trevor_icon_b' : 'shop_trevor_icon_a';
            default:
                return '';
        }
    }
    IsBagdeWhiteSprite(badge) {
        switch (badge) {
            case enums_1.BadgeStyle.Lock:
            case enums_1.BadgeStyle.Tick:
            case enums_1.BadgeStyle.Crown:
                return true;
            default:
                return false;
        }
    }
    BadgeToColor(badge, selected) {
        switch (badge) {
            case enums_1.BadgeStyle.Lock:
            case enums_1.BadgeStyle.Tick:
            case enums_1.BadgeStyle.Crown:
                return selected ? new utils_1.Color(255, 0, 0, 0) : new utils_1.Color(255, 255, 255, 255);
            default:
                return new utils_1.Color(255, 255, 255, 255);
        }
    }
}
UIMenuItem.DefaultBackColor = utils_1.Color.Empty;
UIMenuItem.DefaultHighlightedBackColor = utils_1.Color.White;
UIMenuItem.DefaultForeColor = utils_1.Color.WhiteSmoke;
UIMenuItem.DefaultHighlightedForeColor = utils_1.Color.Black;
exports.UIMenuItem = UIMenuItem;
