"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class Vector3 {
    constructor(x, y, z) {
        this.x = x;
        this.y = y;
        this.z = z;
    }
    static Create(v1) {
        return new Vector3(v1.x, v1.y, v1.z);
    }
    static Clone(v1) {
        return new Vector3(v1.x, v1.y, v1.z);
    }
    static Add(v1, v2) {
        return new Vector3(v1.x + v2.x, v1.y + v2.y, v1.z + v2.z);
    }
    static Subtract(v1, v2) {
        return new Vector3(v1.x - v2.x, v1.y - v2.y, v1.z - v2.z);
    }
    static Multiply(v1, v2) {
        if (typeof v2 === 'number') {
            return new Vector3(v1.x * v2, v1.y * v2, v1.z * v2);
        }
        return new Vector3(v1.x * v2.x, v1.y * v2.y, v1.z * v2.z);
    }
    static Divide(v1, v2) {
        if (typeof v2 === 'number') {
            return new Vector3(v1.x / v2, v1.y / v2, v1.z / v2);
        }
        return new Vector3(v1.x / v2.x, v1.y / v2.y, v1.z / v2.z);
    }
    static DotProduct(v1, v2) {
        return v1.x * v2.x + v1.y * v2.y + v1.z * v2.z;
    }
    static CrossProduct(v1, v2) {
        return new Vector3(v1.y * v2.z - v1.z * v2.y, v1.z * v2.x - v1.z * v2.z, v1.x * v2.y - v1.z * v2.x);
    }
    static Normalize(v) {
        const d = v.DistanceSquared(v);
        return new Vector3(v.x / d, v.y / d, v.z / d);
    }
    clone() {
        return new Vector3(this.x, this.y, this.z);
    }
    DistanceSquared(v) {
        return Math.sqrt(this.x * v.x + this.y * v.y + this.z + v.z);
    }
    Normalize(v) {
        const d = this.DistanceSquared(v);
        return new Vector3(v.x / d, v.y / d, v.z / d);
    }
    CrossProduct(v) {
        return new Vector3(this.y * v.z - this.z * v.y, this.z * v.x - this.z * v.z, this.x * this.y - this.z * v.x);
    }
    Add(v) {
        return new Vector3(this.x + v.x, this.y + v.y, this.z + v.z);
    }
    Subtract(v) {
        return new Vector3(this.x - v.x, this.y - v.y, this.z - v.z);
    }
    Multiply(v) {
        if (typeof v === 'number') {
            return new Vector3(this.x * v, this.y * v, this.z * v);
        }
        return new Vector3(this.x * v.x, this.y * v.y, this.z * v.z);
    }
    Divide(v) {
        if (typeof v === 'number') {
            return new Vector3(this.x / v, this.y / v, this.z / v);
        }
        return new Vector3(this.x / v.x, this.y / v.y, this.z / v.z);
    }
    Replace(v) {
        this.x = v.x;
        this.y = v.y;
        this.z = v.z;
    }
}
exports.Vector3 = Vector3;
